package com.ruoyi.dxp.service;

import java.util.List;
import com.ruoyi.dxp.domain.ServiceItem;

/**
 * 服务字段Service接口
 * 
 * @author jy
 * @date 2022-11-29
 */
public interface IServiceItemService 
{
    /**
     * 查询服务字段
     * 
     * @param id 服务字段主键
     * @return 服务字段
     */
    public ServiceItem selectServiceItemById(Long id);

    /**
     * 查询服务字段列表
     * 
     * @param serviceItem 服务字段
     * @return 服务字段集合
     */
    public List<ServiceItem> selectServiceItemList(ServiceItem serviceItem);

    /**
     * 新增服务字段
     * 
     * @param serviceItem 服务字段
     * @return 结果
     */
    public int insertServiceItem(ServiceItem serviceItem);

    /**
     * 修改服务字段
     * 
     * @param serviceItem 服务字段
     * @return 结果
     */
    public int updateServiceItem(ServiceItem serviceItem);

    /**
     * 批量删除服务字段
     * 
     * @param ids 需要删除的服务字段主键集合
     * @return 结果
     */
    public int deleteServiceItemByIds(Long[] ids);

    /**
     * 删除服务字段信息
     * 
     * @param id 服务字段主键
     * @return 结果
     */
    public int deleteServiceItemById(Long id);

    /**
     * 通过类型和服务id删除服务明细
     * @param serviceType 服务类型
     * @param serviceId 服务id
     * @return
     */
    public void deleteServiceItemByTypeAndDataId(String serviceType, Long serviceId);
}
